# Copyright (c) 2014 Dave Love, Liverpool University
# Licence: BSD

# This implements something effectively similar to Debian's handling
# of linear algebra libraries
# <https://wiki.debian.org/DebianScience/LinearAlgebraLibraries> to
# hack around the Fedora mess which might, say, mean four
# implementations contending in an R application using assorted
# libraries (Rblas, reference (essentially the same), openblas, atlas)
# depending on how things were built.

%global sover 3
%global gslsover 0

Name:		openblas-compat
Version:	1.0
Release:	4%{?dist}
Summary:	Openblas compatibility libraries for reference blas/lapack etc.
# It's not clear it's copyrightable, but inherit the openblas licence.
License:	BSD
BuildRequires:	openblas-devel
BuildArch:	%ix86 x86_64
Group:		System Environment/Libraries
BuildRoot:	%(mktemp -ud %{_tmppath}/%{name}-%{version}-%{release}-XXXXXX)

%description

This package provides trivial shared LAPACK/BLAS libraries which can
replace those from the reference blas, lapack, atlas, gsl, and R
packages at run time so that binaries dynamically linked against those
automatically get the benefit of the generally-faster openblas
routines.

This package can be installed in parallel with the reference versions
and other.  It overrides them via ldconfig configuration in the
absence of a policy of using alternatives for BLAS/LAPACK in Fedora.
(Note that the older atlas in EPEL already overrides the reference
versions in the same way.)  Use LD_LIBRARY_PATH to pick up the
original versions for testing.

On Sandy Bridge with vanilla RHEL6.6/EPEL packages, running the
linpack benchmark single-threaded, openblas is asymptotically around
nine times faster than reference lapack/blas and four times faster
than atlas-sse3 (the best variant for SB that is packaged).
%if 0%{?__isa_bits} == 64
Versions with both 32-bit and 64-bit indexing are included.
%endif


%prep
rm -rf %name-%version
mkdir %name-%version

%build

cd %name-%version
# We need libraries with the right filename and soname, so build
# trivial ones which just pull in openblas.  (Debian builds
# non-trivial versions as alternatives.)
%if 0%{?__isa_bits} == 64
%global lib64s blas64 lapack64
%else
%global lib64s %nil
%endif
# ATLAS changed at 3.10
%if 0%{?el6} || 0%{?el5}
%global atlibs cblas clapack f77blas
%else
%global atlibs satlas
%endif
for n in blas lapack lapacke %lib64s %atlibs; do
  cc -shared -o lib$n.so.%sover -Wl,-soname,lib$n.so.%sover -lopenblas
done
cc -shared -o libgslcblas.so.%gslsover -Wl,-soname,libgslcblas.so.%gslsover -lopenblas

# atlas threaded
%if 0%{?fedora} > 20
cc -shared -o libtatlas.so.%sover -Wl,-soname,libtatlas.so.%sover -lopenblasp -pthread
%else
for n in ptf77blas ptcblas; do
  cc -shared -o lib$n.so.%sover -Wl,-soname,lib$n.so.%sover -lopenblasp -pthread
done
%endif

# R sanctions linking optimized libraries to these in its library
# area, but that gets a spurious link made to libopenblas.so by
# ldconfig and this is cleaner and robust through package updates.
cc -shared -o libRblas.so -lopenblas
cc -shared -o libRlapack.so -lopenblas


%install
rm -rf %buildroot
cd %name-%version
mkdir -p %buildroot%_libdir/openblas-compat
mkdir -p %buildroot%_sysconfdir/ld.so.conf.d
install -m 755 *.so* %buildroot%_libdir/openblas-compat
# needs to collate before atlas*
echo %_libdir/openblas-compat > %buildroot%_sysconfdir/ld.so.conf.d/0openblas-compat-%_arch.conf

%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig

%clean
rm -rf %buildroot


%files
%_libdir/openblas-compat
%config(noreplace) %_sysconfdir/ld.so.conf.d/*


%changelog
* Mon Apr 11 2015 Dave Love <loveshack@fedoraproject.org> - 1.0-4
- Fix condition for atlas choice

* Tue Feb 17 2015 Dave Love <d.love@liverpool.ac.uk> - 1.0-3
- Fix gsl soversion
- Support recent Fedora atlas version
- Add R libs

* Wed Jan 28 2015 Dave Love <d.love@liverpool.ac.uk> - 1.0-2
- Make a build direectory
- Add gslcblas

* Fri Dec 19 2014 Dave Love <d.love@liverpool.ac.uk> - 1.0-1
- Initial packaging
